; Palette Editor Version 2 Copyright (c) 2009 Andrew Owen

	org 50000

; === Jump Table ===

; 50000
	jp	import			; transfer the palette registers to RAM
; 50003
	jp	export			; set the palette registers from RAM
; 50006
	jp	generate		; expand RAM registers to GRB colours
; 50009
	jp	show_edit_screen	; display the edit screen
; 50012
	jp	show_view_screen	; display the view screen
; 50015
	jp	copy_view_screen	; copy the view screen to RAM
; 50018
	jp	load_screen		; load a SCREEN$ file to RAM
; 50021
	jp	save			; save a palette
; 50024
	jp	load			; load a palette

; === Palette Table ===

; 50027

palette_table:
	db	"GRB", "GRB", "GRB", "GRB", "GRB", "GRB", "GRB", "GRB"
	db	"GRB", "GRB", "GRB", "GRB", "GRB", "GRB", "GRB", "GRB"
	db	"GRB", "GRB", "GRB", "GRB", "GRB", "GRB", "GRB", "GRB"
	db	"GRB", "GRB", "GRB", "GRB", "GRB", "GRB", "GRB", "GRB"
	db	"GRB", "GRB", "GRB", "GRB", "GRB", "GRB", "GRB", "GRB"
	db	"GRB", "GRB", "GRB", "GRB", "GRB", "GRB", "GRB", "GRB"
	db	"GRB", "GRB", "GRB", "GRB", "GRB", "GRB", "GRB", "GRB"
	db	"GRB", "GRB", "GRB", "GRB", "GRB", "GRB", "GRB", "GRB"


; === Import Registers ===

import:
	di				; disable interrupts
	ld	hl, registers		; HL = address of registers
	xor	a			; first register

getreg:
	ld	bc, 0xbf3b		; choose register port
	out	(c), a			; select register
	ex	af, af'			; save current register select
	ld	b, 0xff			; choose data port
	in	a, (c)			; get it
	ld	(hl), a			; store it
	ex	af, af'			; restore current register
	inc	hl			; advance pointer
	inc	a			; increase register
	cp	64			; are we nearly there yet?
	jr	nz, getreg		; repeat until all 64 have been done
	ei				; enable interrupts
	ret				; return

; === Export Registers ===

export:
	di				; disable interrupts
	ld	hl, registers		; HL = address of registers
	jp	export2			; continue into similar code


; === Generate Palette Table From Registers ===

generate:
	ld	hl, registers
	ld	de, palette_table
	ld	b, 64

expand:
	ld	a, (hl)		; get the register
	rlca			; rotate right five times to get the green value
	rlca			; 
	rlca			; 
	and	%00000111	; mask out the other bits
	ld	(de), a		; store it
	inc	de		; increment de
	ld	a, (hl)		; get the register
	rra			; rotate twice to get the red value 
	rra			; 
	and	%00000111	; mask out the other bits
	ld	(de), a		; store it
	inc	de		; increment de
	ld	a, (hl)		; get the register
	and	%00000011	; mask out the green and red values
	ld	(de), a		; store the blue value
	inc	de		; increment de
	inc	hl		; increment hl
	djnz	expand		; loop for all registers
	ret			; return


; === Screen Blitting Routines ===

show_edit_screen:
	di				; disable interrupts
	ld	hl, editor_screen	; source
show_2:
	ld	de, 0x4000		; destination
show_3:
	ld	bc, 0x1b00		; bytes
	ldir				; copy	
	ei				; enable interrupts
	ret				; return

show_view_screen:
	di				; disable interrupts
	ld	hl, view_screen		; source
	jr	show_2			;

copy_view_screen:
	di				; disable interrupts
	ld	hl, 0x4000		; source
	ld	de, view_screen		; destination
	jr	show_3			;


; === Screen LOAD Routine ===

load_screen:
	xor	a			; set a to 0 - indicate a header
	scf				; set carry flag - indicate loading
	ld	de, 0x0011		; header is 17 bytes long
	ld	ix, buffer		; address of header data
	call 	0x0556			; input from tape routine
	ld	a, 0xff			; indicate data
	scf				; set carry flag - indicate loading
	ld	de, 0x1b00		; data length
	ld	ix, view_screen		; address of data
	call	0x0556			; input from tape routine
	ret				; return


; === Palette LOAD/SAVE Routines ===

save:
	xor	a			; set a to 0 - indicate a header
					; clear carry flag - indicate saving
	ld	de, 0x0011		; header is 17 bytes long
	ld	ix, header		; address of header data
	call 	0x04c2			; output to tape routine
	ld	a, 0xff			; indicate data
	ld	de, 0x0097		; data length
	ld	ix, basic		; address of data
	call	0x04c2			; output to tape routine
	ret				; return

load:
	xor	a			; set a to 0 - indicate a header
	scf				; set carry flag - indicate loading
	ld	de, 0x0011		; header is 17 bytes long
	ld	ix, buffer		; address of header data
	call 	0x0556			; input from tape routine
	ld	a, 0xff			; indicate data
	scf				; set carry flag - indicate loading
	ld	de, 0x0097		; data length
	ld	ix, basic		; address of data
	call	0x0556			; input from tape routine
	ret				; return

; === Buffer ===

; used for storing the header of palettes and SCREEN$ during loading.

buffer:
	db	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
	db	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
	db	0x00

; === Palette File ===

header:
	db	0x00			; program file
	db	0x14, 0x01, "64colour"	; file name
	dw	0x0097			; data length
	dw	0x0000			; autostart line
	dw	0x0097			; program length

basic:

;    0 RANDOMIZE USR ((PEEK VAL "2
; 3635"+VAL "256"*PEEK VAL "23636"
; )+VAL "48"): LOAD "": REM

	db	0x00, 0x00, 0x93, 0x00, 0xf9, 0xc0, 0x28, 0x28
	db	0xbe, 0xb0, 0x22, 0x32, 0x33, 0x36, 0x33, 0x35
	db	0x22, 0x2b, 0xb0, 0x22, 0x32, 0x35, 0x36, 0x22
	db	0x2a, 0xbe, 0xb0, 0x22, 0x32, 0x33, 0x36, 0x33
	db	0x36, 0x22, 0x29, 0x2b, 0xb0, 0x22, 0x34, 0x38
	db	0x22, 0x29, 0x3a, 0xef, 0x22, 0x22, 0x3a, 0xea


start:
	di				; disable interrupts
	ld	hl, 38			; HL = length of code
	add	hl, bc			; BC = entry point (start) from BASIC

export2:
	ld	bc, 0xbf3b		; register select
	ld	a, 64			; mode group
	out	(c), a			;
	ld	a, 1			; 
	ld	b, 0xff			; choose register port
	out	(c), a			; turn palette mode on
	xor	a			; first register

setreg:
	ld	b, 0xbf			; choose register port
	out	(c), a			; select register
	ex	af, af'			; save current register select
	ld	a, (hl)			; get data
	ld	b, 0xff			; choose data port
	out	(c), a			; set it
	ex	af, af'			; restore current register
	inc	hl			; advance pointer
	inc	a			; increase register
	cp	64			; are we nearly there yet?
	jr	nz, setreg		; repeat until all 64 have been done
	ei				; enable interrupts
	ret				; return

registers:
	db	0x00, 0x02, 0x18, 0x1b, 0xc0, 0xc3, 0xd8, 0xdb	; INK
	db	0x00, 0x02, 0x18, 0x1b, 0xc0, 0xc3, 0xd8, 0xdb	; PAPER
	db	0x00, 0x03, 0x1c, 0x1f, 0xe0, 0xe3, 0xfc, 0xff	; +BRIGHT
	db	0x00, 0x03, 0x1c, 0x1f, 0xe0, 0xe3, 0xfc, 0xff	;
	db	0xdb, 0xd8, 0xc3, 0xc0, 0x1b, 0x18, 0x02, 0x00	; +FLASH
	db	0xdb, 0xd8, 0xc3, 0xc0, 0x1b, 0x18, 0x02, 0x00	;
	db	0xff, 0xfc, 0xe3, 0xe0, 0x1f, 0x1c, 0x03, 0x00	; +BRIGHT/
	db	0xff, 0xfc, 0xe3, 0xe0, 0x1f, 0x1c, 0x03, 0x00	; +FLASH


terminting_byte:
	db	0x0d


; === Editor Screen ===

editor_screen:
	incbin	"editor.scr"

; === View Screen ===

view_screen:
	incbin	"demo.scr"
